import math
import pandas as pd


class MoviesDB:
    def __init__(self):
        self.movies_database = None

    def read_file(self):
        self.movies_database = pd.read_csv("IMDB Top 250 Movies.csv")
        return self.movies_database


class MoviesControl:
    def __init__(self):
        self.link = None
        self.MDdf = None
        self.movies_database = pd.DataFrame(MoviesDB().read_file())

    def search_movies(self, option, value):
        self.MDdf = self.movies_database.copy()
        if option.lower() == "name":
            if value not in self.MDdf['name'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['name'] == value]
                return self.MDdf

        elif option.lower() == "year":
            self.MDdf = self.movies_database.copy()
            # if value type is not int --> change to int and find the year
            # but if the value cannot be changed to int --> return None
            try:
                value = int(value)
            except ValueError:
                return None
            if value not in self.MDdf['year'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['year'] == value]
                return self.MDdf

        elif option.lower() == "genre":
            self.MDdf = self.movies_database.copy()
            # separate the genre column into a list of genres
            self.MDdf['genre'] = self.MDdf['genre'].str.split(',')
            # explode the list of genres into rows
            self.MDdf = self.MDdf.explode('genre')
            # select the rows that contain the genre
            if value not in self.MDdf['genre'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['genre'] == value]
                return self.MDdf

        elif option.lower() == "certificate":
            self.MDdf = self.movies_database.copy()
            if value not in self.MDdf['certificate'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['certificate'] == value]
                return self.MDdf

        elif option.lower() == "casts":
            self.MDdf = self.movies_database.copy()
            self.MDdf['casts'] = self.MDdf['casts'].str.split(',')
            self.MDdf = self.MDdf.explode('casts')
            if value not in self.MDdf['casts'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['casts'] == value]
                return self.MDdf

        elif option.lower() == "directors":
            self.MDdf = self.movies_database.copy()
            self.MDdf['directors'] = self.MDdf['directors'].str.split(',')
            self.MDdf = self.MDdf.explode('directors')
            if value not in self.MDdf['directors'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['directors'] == value]
                return self.MDdf

        elif option.lower() == "writers":
            self.MDdf = self.movies_database.copy()
            self.MDdf['writers'] = self.MDdf['writers'].str.split(',')
            self.MDdf = self.MDdf.explode('writers')
            if value not in self.MDdf['writers'].values:
                return None
            else:
                self.MDdf = self.MDdf.loc[self.MDdf['writers'] == value]
                return self.MDdf

    def get_movie_link(self, movie_name):
        # if the movie name is not in the database --> return None
        if movie_name not in self.MDdf['name'].values:
            return None
        else:
            # get the link of the movie from its name
            self.link = self.MDdf.loc[self.MDdf['name'] == movie_name, 'link'].iloc[0]
            return self.link

    def sorted_movies(self, option):
        def convert_run_time(run_time_str):
            # convert the run_time column to int type; ex. from 2h 28m to 148
            if 'h' in run_time_str and 'm' in run_time_str:
                hours, minutes = run_time_str.split('h ')
                minutes = minutes[:-1]  # remove the 'm' at the end
                return int(hours) * 60 + int(minutes)
            elif 'h' in run_time_str:
                hours = run_time_str[:-1]  # remove the 'h' at the end
                return int(hours) * 60
            else:
                return int(run_time_str[:-1])

        # sort the movies by the option; name, year, rating, run_time, budget, box_office
        self.MDdf = self.movies_database.copy()

        if option.lower() == "name":
            # sort the movies by name from A to Z
            self.MDdf = self.MDdf.sort_values(by=['name'], ascending=True)
            return self.MDdf

        elif option.lower() == "year":
            self.MDdf = self.MDdf.sort_values(by=['year'], ascending=True)
            return self.MDdf

        elif option.lower() == "rating":
            self.MDdf = self.MDdf.sort_values(by=['rating'], ascending=False)
            return self.MDdf

        elif option.lower() == "run_time":
            # sorted by minutes but return the run_time in hours and minutes
            self.MDdf['run_time'] = self.MDdf['run_time'].apply(convert_run_time)
            self.MDdf = self.MDdf.sort_values(by=['run_time'], ascending=True)
            self.MDdf['run_time'] = self.MDdf['run_time'].apply(lambda x: str(int(x / 60)) + 'h ' + str(x % 60) + 'm')
            return self.MDdf

        elif option.lower() == "budget":
            self.MDdf['budget'] = self.MDdf['budget'].replace('Not Available', float('NaN'))
            self.MDdf['budget'] = self.MDdf['budget'].astype(float)
            self.MDdf = self.MDdf.sort_values('budget', ascending=False)
            # Move NaN values to the bottom
            self.MDdf = self.MDdf.sort_values('budget', na_position='last')
            # change the type of the budget to no decimal point and return NaN values to 'Not Available'
            self.MDdf['budget'] = self.MDdf['budget'].apply(
                lambda x: '{:.0f}'.format(x) if not math.isnan(x) else 'Not Available')
            return self.MDdf

        elif option.lower() == "box_office":
            self.MDdf['box_office'] = self.MDdf['box_office'].replace('Not Available', float('NaN'))
            self.MDdf['box_office'] = self.MDdf['box_office'].astype(float)
            self.MDdf = self.MDdf.sort_values('box_office', ascending=False)
            # Move NaN values to the bottom
            self.MDdf = self.MDdf.sort_values('box_office', na_position='last')
            # change the type of the box_office to no decimal point and return NaN values to 'Not Available'
            self.MDdf['box_office'] = self.MDdf['box_office'].apply(
                lambda x: '{:.0f}'.format(x) if not math.isnan(x) else 'Not Available')
            return self.MDdf


class MoviesStatistics:
    def __init__(self):
        self.link = None
        self.MDdf = None
        self.movies_database = pd.DataFrame(MoviesDB().read_file())

    def bar_graph(self, column):
        def convert_run_time(run_time_str):
            # convert the run_time column to int type; ex. from 2h 28m to 148
            if 'h' in run_time_str and 'm' in run_time_str:
                hours, minutes = run_time_str.split('h ')
                minutes = minutes[:-1]  # remove the 'm' at the end
                return int(hours) * 60 + int(minutes)
            elif 'h' in run_time_str:
                hours = run_time_str[:-1]  # remove the 'h' at the end
                return int(hours) * 60
            else:
                return int(run_time_str[:-1])

        # column of bar = ['year', 'rating', 'run_time', 'budget', 'box_office']
        if column == 'year':
            # get the number of movies in each year
            self.MDdf = self.movies_database.copy()
            self.MDdf = self.MDdf['year'].value_counts()
            return self.MDdf
        elif column == 'rating':
            # get their average rating in each year
            self.MDdf = self.movies_database.copy()
            self.MDdf = self.MDdf.groupby('year')['rating'].mean()
            return self.MDdf
        elif column == 'run_time':
            # get their average run_time in each year
            self.MDdf = self.movies_database.copy()
            self.MDdf['run_time'] = self.MDdf['run_time'].apply(lambda x: convert_run_time(x))
            self.MDdf = self.MDdf.groupby('year')['run_time'].mean()
            return self.MDdf

    def pie_graph(self, column):
        # column of pie = ['certificate', 'genre']
        if column == 'certificate':
            self.MDdf = self.movies_database.copy()
            self.MDdf = self.MDdf['certificate'].value_counts()
            return self.MDdf

        elif column == 'genre':
            self.MDdf = self.movies_database.copy()
            self.MDdf['genre'] = self.MDdf['genre'].str.split(',')
            self.MDdf = self.MDdf.explode('genre')
            self.MDdf = self.MDdf['genre'].value_counts()
            return self.MDdf

    def line_graph(self, column):
        # column of line = = ['budget', 'box_office']
        if column == 'budget':
            # get the trend of average budget in each year
            self.MDdf = self.movies_database.copy()
            self.MDdf['budget'] = self.MDdf['budget'].replace('Not Available', float(0))
            self.MDdf['budget'] = self.MDdf['budget'].astype(float)
            self.MDdf = self.MDdf.groupby('year')['budget'].mean()
            return self.MDdf

        if column == 'box_office':
            # get the trend of maximum box_office in each year
            self.MDdf = self.movies_database.copy()
            self.MDdf['box_office'] = self.MDdf['box_office'].replace('Not Available', float(0))
            self.MDdf['box_office'] = self.MDdf['box_office'].astype(float)
            self.MDdf = self.MDdf.groupby('year')['box_office'].max()
            return self.MDdf

    def scatter_graph(self):
        def convert_run_time(run_time_str):
            # convert the run_time column to int type; ex. from 2h 28m to 148
            if 'h' in run_time_str and 'm' in run_time_str:
                hours, minutes = run_time_str.split('h ')
                minutes = minutes[:-1]  # remove the 'm' at the end
                return int(hours) * 60 + int(minutes)
            elif 'h' in run_time_str:
                hours = run_time_str[:-1]  # remove the 'h' at the end
                return int(hours) * 60
            else:
                return int(run_time_str[:-1])

        # x-column = ['rating', 'run_time', 'budget', 'box_office']
        # y-column = ['rating', 'run_time', 'budget', 'box_office']
        self.MDdf = self.movies_database.copy()
        self.MDdf['run_time'] = self.MDdf['run_time'].apply(lambda x: convert_run_time(x))
        self.MDdf['budget'] = self.MDdf['budget'].replace('Not Available', float('NaN'))
        self.MDdf['budget'] = self.MDdf['budget'].astype(float)
        self.MDdf['box_office'] = self.MDdf['box_office'].replace('Not Available', float('NaN'))
        self.MDdf['box_office'] = self.MDdf['box_office'].astype(float)
        return self.MDdf
