from tkinter import *
import tkinter as tk
import seaborn as sns
from PIL import ImageTk, Image
from tkinter import messagebox
from tkinter import ttk
from matplotlib import pyplot as plt
from matplotlib.backends._backend_tk import NavigationToolbar2Tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from MoviesDB import MoviesControl
from MoviesDB import MoviesStatistics
from MoviesDB import MoviesDB
import webview

class Page(tk.Tk):
    def __init__(self):
        super().__init__()
        # show window
        self.title("IMDB's Hall of Fame Program")
        self.geometry("960x540")
        self.iconbitmap("icon.ico")

        # set background image
        self.bg = ImageTk.PhotoImage(file="bg.png")
        self.canvas = tk.Canvas(self, width=960, height=540)
        self.canvas.pack(fill="both", expand=True)
        self.canvas.create_image(0, 0, image=self.bg, anchor="nw")

        self.maxsize(960, 540)
        self.minsize(960, 540)

        self.menu = tk.Menu(self)
        self.config(menu=self.menu)
        self.menu.add_command(label="About", command=self.aboutProgram)
        self.menu.add_command(label="Exit", command=self.exitProgram)

        self.btn_menu1 = tk.Button(self, text="Movie Information Page", state="normal", font='Arial 10 bold', background='white', foreground='black', command=self.openInformationPage)
        self.btn_menu1.place(x=275, y=350)
        self.btn_menu2 = tk.Button(self, text="Movie Statistics Page", state="normal", font='Arial 10 bold', background='white', foreground='black', command=self.openStatisticsPage)
        self.btn_menu2.place(x=500, y=350)

    def aboutProgram(self):
        messagebox.showinfo("About",
                            "This Program is about the top movies from IMDB and it is the part of the year project.\n\n"
                            "Thank you for using this program(◍•ᴗ•◍)❤.")

    def exitProgram(self):
        if tk.messagebox.askyesno("Exit", "Do you want to exit?"):
            self.destroy()

    def openInformationPage(self):
        self.destroy()
        InformationPage()

    def openStatisticsPage(self):
        self.destroy()
        StatisticsPage()

    def run(self):
        self.mainloop()


class InformationPage(Page):
    def __init__(self):
        super().__init__()
        self.link = None
        self.moviesCTRL = MoviesControl()

        # change menu state
        self.btn_menu1.config(state="disabled")
        # change position of menu button
        self.btn_menu1.place(x=275, y=500)
        self.btn_menu2.config(state="normal")
        self.btn_menu2.place(x=500, y=500)

        self.lb2 = tk.Label(self, text="Movie Information Page", background='#1F222B', foreground='white',
                            font='Arial 20 bold')
        self.lb2.place(x=325, y=50)

        # frame1 for search and sort
        self.frame1 = tk.Frame(self, width=700, height=120, background='#1F222B')
        self.frame1.place(x=125, y=100)

        # combo box for search
        self.choice_search = tk.StringVar(value="Choose an option")
        self.combo_search = ttk.Combobox(self.frame1, textvariable=self.choice_search,
                                         values=["name", "year", "genre", "certificate", "casts", "directors",
                                                 "writers"],
                                         width=20, font="10", state="readonly")
        self.combo_search.place(x=10, y=40)

        # entry for search
        self.search_entry = tk.Entry(self.frame1, width=32, font="10")
        self.search_entry.place(x=270, y=40)

        self.btn_search = tk.Button(self.frame1, text="Search",
                                    command=lambda: self.search(self.combo_search.get(), self.search_entry.get()))
        self.btn_search.place(x=640, y=40)
        self.btn_clear = tk.Button(self.frame1, text="Clear",
                                   command=lambda: self.clear_search())
        self.btn_clear.place(x=400, y=80)

        self.lb_sort = tk.Label(self.frame1, text="Sort by:", background='#1F222B', foreground='white',
                                font='Arial 12 ')
        self.lb_sort.place(x=10, y=80)

        # combo box for sort
        self.choice_sort = tk.StringVar(value="Choose an option")
        self.combo_sort = ttk.Combobox(self.frame1, textvariable=self.choice_sort,
                                       values=["name", "year", "rating", "run_time", "budget", "box_office"],
                                       width=20, font="10", state="readonly")
        self.combo_sort.place(x=80, y=80)
        self.btn_show = tk.Button(self.frame1, text="Show", command=lambda: self.show_movies())
        self.btn_show.place(x=350, y=80)

        # frame2 for movie information
        self.frame2 = tk.Frame(self, background='#1F222B')
        self.frame2.place(x=125, y=220, width=700, height=250)

        self.lb_search = tk.Label(self.frame1, text="Search by:", background='#1F222B', foreground='white'
                                  , font='Arial 12 ')
        self.lb_search.place(x=10, y=10)

    def show_movies(self):
        try:
            self.tree.destroy()
            self.scrolly.destroy()
            self.scrollx.destroy()
        except:
            pass
        data = self.moviesCTRL.sorted_movies(self.combo_sort.get())
        if data is None:
            # destroy treeview widget if it exists
            try:
                self.tree.destroy()
                self.scrolly.destroy()
                self.scrollx.destroy()
            except:
                pass
            self.lb_not_found = tk.Label(self.frame2, text="Please select sort option first", background='#1F222B',
                                         foreground='white', font='Arial 12 bold')
            self.lb_not_found.place(x=250, y=100)
        else:  # show treeview
            # destroy not found label if it exists
            try:
                self.lb_not_found.destroy()
            except:
                pass
            # destroy treeview widget if it exists
            try:
                self.tree.destroy()
                self.scrolly.destroy()
                self.scrollx.destroy()
            except:
                pass
            # create treeview widget
            self.tree = ttk.Treeview(self.frame2)
            self.tree.place(relheight=1, relwidth=1)
            self.scrolly = tk.Scrollbar(self.frame2, orient=tk.VERTICAL, command=self.tree.yview)
            self.scrolly.pack(side=tk.RIGHT, fill=tk.Y)
            self.scrollx = tk.Scrollbar(self.frame2, orient=tk.HORIZONTAL, command=self.tree.xview)
            self.scrollx.pack(side=tk.BOTTOM, fill=tk.X)
            self.tree.configure(xscrollcommand=self.scrollx.set, yscrollcommand=self.scrolly.set)

            self.tree["column"] = list(data.columns[0:13])
            self.tree["show"] = "headings"
            for column in self.tree["column"]:
                self.tree.heading(column, text=column)
            df_rows = data.to_numpy().tolist()
            for row in df_rows:
                self.tree.insert("", "end", values=row)
            self.tree.bind("<Double-1>", self.click)

    def search(self, search_by, search_entry):
        # destroy treeview widget if it exists
        try:
            self.tree.destroy()
        except:
            pass
        # search movies and display in treeview
        data = self.moviesCTRL.search_movies(search_by, search_entry)
        if data is None:
            # destroy treeview widget if it exists
            try:
                self.tree.destroy()
                self.scrolly.destroy()
                self.scrollx.destroy()
            except:
                pass
            self.lb_not_found = tk.Label(self.frame2, text="Sorry, this movie is not found", background='#1F222B',
                                         foreground='white', font='Arial 12 bold')
            self.lb_not_found.place(x=250, y=100)
        else:  # show treeview
            # destroy not found label if it exists
            try:
                self.lb_not_found.destroy()
            except:
                pass
            # destroy treeview widget if it exists
            try:
                self.tree.destroy()
                self.scrolly.destroy()
                self.scrollx.destroy()
            except:
                pass
            # create treeview widget
            self.tree = ttk.Treeview(self.frame2)
            self.tree.place(relheight=1, relwidth=1)
            self.scrolly = tk.Scrollbar(self.frame2, orient=tk.VERTICAL, command=self.tree.yview)
            self.scrolly.pack(side=tk.RIGHT, fill=tk.Y)
            self.scrollx = tk.Scrollbar(self.frame2, orient=tk.HORIZONTAL, command=self.tree.xview)
            self.scrollx.pack(side=tk.BOTTOM, fill=tk.X)
            self.tree.configure(xscrollcommand=self.scrollx.set, yscrollcommand=self.scrolly.set)

            self.tree['column'] = list(data.columns[0:13])
            self.tree['show'] = 'headings'
            for column in self.tree['columns']:
                self.tree.heading(column, text=column)
            data_rows = data.to_numpy().tolist()
            for row in data_rows:
                self.tree.insert('', 'end', values=row)
            self.tree.bind("<Double-1>", self.click)

    def click(self, event):
        # get the movie name from the selected row
        selected_item = self.tree.focus()
        if selected_item:
            item_data = self.tree.item(selected_item)['values']
            movie_name = item_data[1]
            # open the link in webview
            self.link = self.moviesCTRL.get_movie_link(movie_name)
            webview.create_window("Movie Information", self.link, width=700, height=300, resizable=True,
                                  fullscreen=False, frameless=False, min_size=(700, 300))
            webview.start()

    def clear_search(self):
        # clear search entry and destroy treeview widget if it exists
        self.search_entry.delete(0, 'end')
        if hasattr(self, 'tree'):
            # destroy treeview widget and their scrollbars
            self.tree.destroy()
            self.scrolly.destroy()
            self.scrollx.destroy()


class StatisticsPage(Page):
    def __init__(self):
        super().__init__()
        self.link = None
        self.moviesCTRL = MoviesControl()
        self.moviesSTAT = MoviesStatistics()

        # change menu state
        self.btn_menu1.config(state="normal")
        self.btn_menu1.place(x=275, y=500)
        self.btn_menu2.config(state="disabled")
        self.btn_menu2.place(x=500, y=500)

        self.lb3 = tk.Label(self, text="Movie Statistics Page", background='#1F222B', foreground='white',
                            font='Arial 20 bold')
        self.lb3.place(x=350, y=50)

        # frame1 for filters
        self.frame1 = tk.Frame(self, width=700, height=350, background='#1F222B')
        self.frame1.place(x=125, y=100)

        self.lb4 = tk.Label(self.frame1, text="Choose graph type:", background='#1F222B', foreground='white',
                            font='Arial 12')
        self.lb4.place(x=10, y=10)

        # combobox for graph type
        self.combo_type = ttk.Combobox(self.frame1, width=20, state='readonly', font=10)
        self.combo_type['values'] = ['Bar', 'Pie', 'Line', 'Scatter']
        self.combo_type.place(x=180, y=10)
        self.combo_type.bind('<<ComboboxSelected>>', self.on_select_graph_type)

        self.btn_show_graph = tk.Button(self.frame1, text="Show Graph", width=15, height=1, background='white',
                                        foreground='#1F222B', font='Arial 12', command=self.show_graph)
        self.btn_show_graph.place(x=520, y=10)

    def on_select_graph_type(self, event):
        graph_type = self.combo_type.get()
        # remove any existing comboboxes
        for child in self.frame1.winfo_children():
            if isinstance(child, ttk.Combobox) and child != self.combo_type:
                child.destroy()

        # create comboboxes for selected graph type
        if graph_type == 'Bar':
            self.lb5 = tk.Label(self.frame1, text="What data that you want to plot:", background='#1F222B',
                                foreground='white', font='Arial 12')
            self.lb5.place(x=10, y=50)

            # create combobox for column selection
            self.combo_column = ttk.Combobox(self.frame1, width=20, state='readonly', font=10)
            self.combo_column['values'] = ['year', 'rating', 'run_time']
            self.combo_column.place(x=250, y=50)

        elif graph_type == 'Pie':
            self.lb5 = tk.Label(self.frame1, text="What data that you want to plot:", background='#1F222B',
                                foreground='white', font='Arial 12')
            self.lb5.place(x=10, y=50)

            # create combobox for column selection
            self.combo_column = ttk.Combobox(self.frame1, width=20, state='readonly', font=10)
            self.combo_column['values'] = ['certificate', 'genre']
            self.combo_column.place(x=250, y=50)

        elif graph_type == 'Line':
            self.lb5 = tk.Label(self.frame1, text="What data that you want to plot:", background='#1F222B',
                                foreground='white', font='Arial 12')
            self.lb5.place(x=10, y=50)

            # create combobox for column selection
            self.combo_column = ttk.Combobox(self.frame1, width=20, state='readonly', font=10)
            self.combo_column['values'] = ['budget', 'box_office']
            self.combo_column.place(x=250, y=50)

        if graph_type == 'Scatter':
            self.lb5 = tk.Label(self.frame1, text="What data that you want to plot:", background='#1F222B',
                                foreground='white', font='Arial 12')
            self.lb5.place(x=10, y=50)

            # create combobox for x-axis selection
            self.combo_x = ttk.Combobox(self.frame1, width=15, state='readonly', font=10)
            self.combo_x['values'] = ['rating', 'run_time', 'budget', 'box_office']
            self.combo_x.place(x=270, y=50)

            # create combobox for y-axis selection
            self.combo_y = ttk.Combobox(self.frame1, width=15, state='readonly', font=10)
            self.combo_y['values'] = ['rating', 'run_time', 'budget', 'box_office']
            self.combo_y.place(x=475, y=50)

    def show_graph(self):
        graph_type = self.combo_type.get()
        if graph_type == 'Bar':
            column_name = self.combo_column.get()
            data = self.moviesSTAT.bar_graph(column_name)
            # plot bar graph form the data
            plt.bar(data.index, data)
            if column_name == 'year':
                plt.title("Numbers of Movies by " + column_name)
                # set x-axis label and y-axis label
                plt.xlabel(column_name)
                plt.ylabel("Number of Movies")
                plt.show()
            elif column_name == 'rating':
                plt.title("Average rating of Movies in each year")
                # set x-axis label and y-axis label
                plt.xlabel("year")
                plt.ylabel("Average rating")
                # set limit for y-axis
                plt.ylim(7, 10)
                plt.show()
            elif column_name == 'run_time':
                plt.title("Average run time of Movies in each year")
                # set x-axis label and y-axis label
                plt.xlabel("year")
                plt.ylabel("Average run time (minutes)")
                # set limit for y-axis
                plt.ylim(50, 200)
                plt.show()

        elif graph_type == 'Pie':
            column_name = self.combo_column.get()
            data = self.moviesSTAT.pie_graph(column_name)
            plt.pie(data, labels=data.index, autopct='%1.1f%%', shadow=True, startangle=90)
            plt.title("Numbers of Movies by " + column_name)
            plt.show()

        elif graph_type == 'Line':
            column_name = self.combo_column.get()
            data = self.moviesSTAT.line_graph(column_name)
            if column_name == 'budget':
                plt.plot(data.index, data)
                plt.title("Average " + column_name + " of Movies in each year")
                plt.xlabel("year")
                plt.ylabel("Average " + column_name + " (USD)")
                plt.show()
            if column_name == 'box_office':
                plt.plot(data.index, data)
                plt.title("Maximum " + column_name + " of Movies in each year")
                plt.xlabel("year")
                plt.ylabel("Maximum " + column_name)
                plt.show()

        elif graph_type == 'Scatter':
            x_column = self.combo_x.get()
            y_column = self.combo_y.get()
            data = self.moviesSTAT.scatter_graph()

            fig, ax = plt.subplots(figsize=(5, 5), dpi=100)
            palette = sns.color_palette('muted', len(data[x_column].unique()))
            ax.scatter(data[x_column], data[y_column],
                       c=data[x_column].map(dict(zip(data[x_column].unique(), palette))))
            ax.set_xlabel(x_column)
            ax.set_ylabel(y_column)
            ax.set_title("Scatter plot between " + x_column + " and " + y_column)
            scatter_window = tk.Toplevel(self)
            scatter_window.title("Scatter Plot")
            scatter_window.geometry("500x500")
            canvas = FigureCanvasTkAgg(fig, master=scatter_window)
            canvas.draw()

            toolbar = NavigationToolbar2Tk(canvas, scatter_window)
            toolbar.update()
            canvas.get_tk_widget().pack()
